Package: Deriv
Type: Package
Title: Symbolic Differentiation
Version: 4.2.0
Date: 2025-06-20
Authors@R: c(person(given="Andrew", family="Clausen", role="aut"),
    person(given="Serguei", family="Sokol", role=c("aut", "cre"),
    email="sokol@insa-toulouse.fr", comment = c(ORCID = "0000-0002-5674-3327")),
    person(given="Andreas", family="Rappold", role="ctb", email="arappold@gmx.at"))
Description: R-based solution for symbolic differentiation. It admits
    user-defined function as well as function substitution
    in arguments of functions to be differentiated. Some symbolic
    simplification is part of the work.
License: GPL (>= 3)
Suggests: testthat (>= 0.11.0)
BugReports: https://github.com/sgsokol/Deriv/issues
RoxygenNote: 7.3.1
Imports: methods
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2025-06-20 13:25:20 UTC; sokol
Author: Andrew Clausen [aut],
  Serguei Sokol [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5674-3327>),
  Andreas Rappold [ctb]
Maintainer: Serguei Sokol <sokol@insa-toulouse.fr>
Repository: CRAN
Date/Publication: 2025-06-20 15:20:02 UTC
Built: R 4.6.0; ; 2026-04-25 15:42:30 UTC; unix
